#ifndef CRBKO_H
#define CRBKO_H

/*
 * Copyright(c) 1992 RWS Dienst Verkeerkunde
 *
 * File         : crbko.h
 *
 * Description  : support functies voor het blokobject
 *                Iedere entry is zodanig opgezet dat plaats- en
 *                volgorde-afhankelijkheid in de tabellen
 *                niet voorkomt.
 *
 * History      : 18-01-90 Creation
 *                6-7-92,  aanpassing aan nieuwe interface definitie
 *                         van de commissie C van de CVN
 *
 */

#include "crbko.d"

extern struct SGBK_PTRS    sgbk[ MAX_SG ];
extern struct BK_PTRS      bk;

/*
 * macro's voor sg- en bk afhankelijke subfuncties
 *
 */
#define         Hfsb_val(a,b,c)          ( sgbk[(b)].b_hfbs[(c)][(a)]&   sgbk[(b)].m)
#define         Hfsb_begin(a,b,c)        ( sgbk[(b)].bg_hfbs[(c)][(a)]&  sgbk[(b)].m)
#define         Hfsb_end(a,b,c)          ( sgbk[(b)].en_hfbs[(c)][(a)]&  sgbk[(b)].m)
#define         Hfsb_tracon(a,b,c)       ( sgbk[(b)].tr_hfbs[(c)][(a)]|= sgbk[(b)].m)
#define         Hfsb_tracof(a,b,c)       ( sgbk[(b)].tr_hfbs[(c)][(a)]&=(Mask16)~(unsigned)sgbk[(b)].m)

/*
 * macro's voor sg- en bk afhankelijke opties
 *
 */
#define         Opsb_val(a,b,c)          ( sgbk[(b)].b_opbs[(c)][(a)]&   sgbk[(b)].m)
#define         Opsb_begin(a,b,c)        ( sgbk[(b)].bg_opbs[(c)][(a)]&  sgbk[(b)].m)
#define         Opsb_end(a,b,c)          ( sgbk[(b)].en_opbs[(c)][(a)]&  sgbk[(b)].m)
#define         Opsb_tracon(a,b,c)       ( sgbk[(b)].tr_opbs[(c)][(a)]|= sgbk[(b)].m)
#define         Opsb_tracof(a,b,c)       ( sgbk[(b)].tr_opbs[(c)][(a)]&=(Mask16)~(unsigned)sgbk[(b)].m)


/*
 * macro's voor bk afhankelijke subfunties
 *
 */
#define         Hfb_val(a,c)             ( bk.b_hfb[(c)][(a)])
#define         Hfb_begin(a,c)           ( bk.bg_hfb[(c)][(a)])
#define         Hfb_end(a,c)             ( bk.en_hfb[(c)][(a)])
#define         Hfb_tracon(a,c)          ( bk.tr_hfb[(c)][(a)] = 1)
#define         Hfb_tracof(a,c)          ( bk.tr_hfb[(c)][(a)] = 0)


/*
 * macro's voor bk afhankelijke opties
 *
 */
#define         Opb_val(a,c)             ( bk.b_opb[(c)][(a)])
#define         Opb_begin(a,c)           ( bk.bg_opb[(c)][(a)])
#define         Opb_end(a,c)             ( bk.en_opb[(c)][(a)])
#define         Opb_tracon(a,c)          ( bk.tr_opb[(c)][(a)]|= 1)
#define         Opb_tracof(a,c)          ( bk.tr_opb[(c)][(a)]&=(Mask16)~(unsigned)1)

/*
 * macro's voor wachtstandrichtingen, primaire en alt. SG 's en wachtblok
 *
 */
#define         Bk_is_pr_sg(b,c)         ( *(sgbk[(b)].p_prm[(c)]) & sgbk[(b)].m)
#define         Bk_is_al_sg(b,c)         ( *(sgbk[(b)].p_alt[(c)]) & sgbk[(b)].m)

/*
 * supportformules
 *
 */
extern  Bool    Bk_init(void);                          /* initialisatie */

extern  void    Hfsb_set(Mask16,Int16,Int16);           /* set hulpfunctie    */
extern  void    Hfsb_reset(Mask16,Int16,Int16);         /* reset hulpfunctie  */

extern  void    Opsb_set(Mask16,Int16,Int16);           /* set optie          */
extern  void    Opsb_reset(Mask16,Int16,Int16);         /* reset optie        */

extern  void    Hfb_set(Mask16,Int16);                  /* set hulpfunctie    */
extern  void    Hfb_reset(Mask16,Int16);                /* reset hulpfunctie  */

extern  void    Opb_set(Mask16,Int16);                  /* set optie          */
extern  void    Opb_reset(Mask16,Int16);                /* reset optie        */


extern  Bool    Bk_all_prsg (Int16 bk,Bool64 sg);       /* alle primaire signaalgroepen     */
extern  Bool    Bk_all_altsg(Int16 bk,Bool64 sg);       /* alle alternatieve signaalgroepen */
extern  Bool    Bk_all_bk_hfsb(Int16 hf, Bool64 sg);    /* alle hf's in alle blokken */
extern  Bool    Bk_all_hfsb(Int16 hf,Int16 blk, Bool64 sg_all);
                                                           /* geef alle SG's waarvan in blok blk de hulpfunctie */
                                                           /* aanstaat */
extern  Bool    Bk_pg(Int16 sg);                           /* deze SG in huidige blok al prim. gerealiseerd? */
extern  Bool    Bk_ar(Int16 sg);                           /* deze SG (in elk blok) in alt. realisatie? */
extern  Bool    Bk_pr(Int16 sg);                           /* deze SG (in elk blok) in prim. realisatie? */
extern  Bool    Bk_get_wb(Int16 *wb);                      /* geef het wachtblok# */
extern  Bool    Bk_wb_akt(Int16 bk);                       /* is bk als wachtblok aktief */
extern  Bool    Bk_hf_in_bl(Int16 hf,Int16 sg, Int16 *bk); /* is een hulpfunctie in een blok gezet */
extern  void    Bk_rfsh(void);                             /* vul "old" waardes in  */
extern  Int16   Bk_next_bk(Int16);                         /* set en geef volgende blok */

extern  Bool    Bk_sg_is_prm(Int16 sg, Int16 blk);         /* is sgn_n primair */
extern  Bool    Bk_sg_is_alt(Int16 sg, Int16 blk);         /* is sgn_n alternatief ? */
extern  Bool    Bk_prsg_bk_mb(Int16 sgn_n, Int16 bk, Int16 *blk_mb);
                                                           /* eerstvolgende blok waarin */
                                                           /* sgn_n primair is */

extern  void    Opb_eval(void);                            /* evalueer blok gebonden functies */

#endif  /* CRBKO_H */
