#ifndef CRDTO_H
#define CRDTO_H

/*
 * Copyright(c) 1992 RWS Dienst Verkeerkunde
 *
 * File         : crdto.h
 *
 * Description  : functies en macro's detector object
 *                Iedere functie die als macro gedefinierd kan worden
 *                wordt ook als macro gedefinieerd.
 *
 * History      : 29-01-90 Creation
 *              : 02-10-90 CCIE-interface aanpassing
 *                6-7-92,  aanpassing aan nieuwe interface definitie
 *                         van de commissie C van de CVN
 *
 */


#include "crdto.d"

extern struct DT_PTRS          dt [ MAX_DET ];


/*
 * macro's voor resp waarde, begin en einde detector
 *
 */

#define Dt_val(a)       ( *(dt[(a)].s) & dt[(a)].m)
#define Dt_begin(a)     ( (Mask16)~(unsigned)(*(dt[(a)].vs)) &  *(dt[(a)].s) & dt[(a)].m)
#define Dt_end(a)       ( *(dt[(a)].vs) & (Mask16)~(unsigned)(*(dt[(a)].s)) & dt[(a)].m)
#define Dt_set(n,m)     ( dt[n].functie|=m)
#define Dt_reset(n,m)   ( dt[n].functie&=(Mask16)~(unsigned)m)

#define Dt_speed_val(a)       ( CIF_IS[CIF_AANT_IS_D+OVERIGE_SIGNALEN+MAX_KOPPELSGN+a] & 0xFF )
#define Dt_speed_direction(a) ( (CIF_IS[CIF_AANT_IS_D+OVERIGE_SIGNALEN+MAX_KOPPELSGN+a] & 0x8000) != 0x00000000 )
#define Dt_speed_status(a)    ( (CIF_IS[CIF_AANT_IS_D+OVERIGE_SIGNALEN+MAX_KOPPELSGN+a] & 0x1000) != 0x00000000 )
#define Dt_speed_vehicle(a)   ( (CIF_IS[CIF_AANT_IS_D+OVERIGE_SIGNALEN+MAX_KOPPELSGN+a] & 0x700) >>8 )

#define Dt_length_val(a)       ( CIF_IS[CIF_AANT_IS_D+OVERIGE_SIGNALEN+MAX_KOPPELSGN+CIF_AANT_IS_D+2*MAX_REGELINGEN+a] & 0x1FFF )
#define Dt_length_direction(a) ( (CIF_IS[CIF_AANT_IS_D+OVERIGE_SIGNALEN+MAX_KOPPELSGN+CIF_AANT_IS_D+2*MAX_REGELINGEN+a] & 0x8000) != 0x00000000 )
#define Dt_length_status(a)    ( (CIF_IS[CIF_AANT_IS_D+OVERIGE_SIGNALEN+MAX_KOPPELSGN+CIF_AANT_IS_D+2*MAX_REGELINGEN+a] & 0x4000) != 0x00000000 )

#define Dt_s_val(a)     ( *(dt[(a)].s_s) & dt[(a)].m)
#define Dt_s_begin(a)   ( (Mask16)~(unsigned)(*(dt[(a)].vs_s)) &  *(dt[(a)].s_s) & dt[(a)].m)
#define Dt_s_end(a)     ( *(dt[(a)].vs_s) & (Mask16)~(unsigned)(*(dt[(a)].s_s)) & dt[(a)].m)
#define Dt_og_val(a)    ( *(dt[(a)].s_og) & dt[(a)].m)
#define Dt_og_begin(a)  ( (Mask16)~(unsigned)(*(dt[(a)].vs_og)) &  *(dt[(a)].s_og) & dt[(a)].m)
#define Dt_og_end(a)    ( *(dt[(a)].vs_og) & (Mask16)~(unsigned)(*(dt[(a)].s_og)) & dt[(a)].m)
#define Dt_bg_val(a)    ( *(dt[(a)].s_bg) & dt[(a)].m)
#define Dt_bg_begin(a)  ( (Mask16)~(unsigned)(*(dt[(a)].vs_bg)) &  *(dt[(a)].s_bg) & dt[(a)].m)
#define Dt_bg_end(a)    ( *(dt[(a)].vs_bg) & (Mask16)~(unsigned)(*(dt[(a)].s_bg)) & dt[(a)].m)
#define Dt_fg_val(a)    ( *(dt[(a)].s_fg) & dt[(a)].m)
#define Dt_fg_begin(a)  ( (Mask16)~(unsigned)(*(dt[(a)].vs_fg)) &  *(dt[(a)].s_fg) & dt[(a)].m)
#define Dt_fg_end(a)    ( *(dt[(a)].vs_fg) & (Mask16)~(unsigned)(*(dt[(a)].s_fg)) & dt[(a)].m)

#define Dt_tracon(a)    ( dt[(a)].tr |= (Mask16) 1 )
#define Dt_tracof(a)    ( dt[(a)].tr &= ~(Mask16)1 )
#define Dt_s_tracon(a)  ( dt[(a)].tr |= (Mask16) 2 )
#define Dt_s_tracof(a)  ( dt[(a)].tr &= ~(Mask16)2 )
#define Dt_og_tracon(a) ( dt[(a)].tr |= (Mask16) 4 )
#define Dt_og_tracof(a) ( dt[(a)].tr &= ~(Mask16)4 )
#define Dt_bg_tracon(a) ( dt[(a)].tr |= (Mask16) 8 )
#define Dt_bg_tracof(a) ( dt[(a)].tr &= ~(Mask16)8 )
#define Dt_fg_tracon(a) ( dt[(a)].tr |= (Mask16) 16 )
#define Dt_fg_tracof(a) ( dt[(a)].tr &= ~(Mask16)16 )
       
#define Dt_td_val(a,b)      ( dt[(b)].b_td[(a)]& dt[(b)].m)
#define Dt_td_begin(a,b)    ( dt[(b)].bg_td[(a)]& dt[(b)].m)
#define Dt_td_end(a,b)      ( dt[(b)].en_td[(a)]& dt[(b)].m)
#define Dt_td_tracon(a,b)   ( dt[(b)].tr_td[(a)]|= dt[(b)].m)
#define Dt_td_tracof(a,b)   ( dt[(b)].tr_td[(a)]&=(Mask16)~(unsigned)dt[(b)].m)
#define Dt_td_wtracon(a,b)  ( dt[(b)].tr_wtd[(a)]|= dt[(b)].m)
#define Dt_td_wtracof(a,b)  ( dt[(b)].tr_wtd[(a)]&=(Mask16)~(unsigned)dt[(b)].m)
#define Dt_td_def(a,b)      ( dt[(b)].p_dt[(a)]!=NULL)        /* tijd gedefinieerd voor deze detector? */
#define Dt_td_halted(a,b)   ( dt[(b)].hl_td[(a)]& dt[(b)].m)  /* tijd gehalteerd ? */

#define Dt_keuze_eh(a)      (*(dt[(a)].p_eh) & dt[(a)].m)     /* extra hiaattijd is gekozen */

/*
 * support procedures voor detectoren
 *
 */
extern  Bool    Dt_init(void);             /* initialisatie */
extern  void    Dt_lees_in(Int16*, Int16); /* lees nieuwe detectorstanden in */
extern  void    Dt_tel_h1_h2(void);        /* hou bij hoeveel h1 en h2 tijden lopen op de signaalgroep */
extern  Bool    Dtbs_val(Int16);           /* is van een SG een detector vlgns BS h5p aktief?   */
extern  Bool    H1e_loopt(Int16);          /* is van een SG een detector met h1 functie aktief? */
extern  Bool    H2e_loopt(Int16);          /* is van een SG een detector met h2 functie aktief? */
extern  Bool    Dtbs_h5o(Int16);           /* hoeveel detectoren hebben H1e functie en gedefinieerde hiaattijd vlgns BS h5o? */
extern  Bool    Dtbs_h5x(Int16);           /* hoeveel detectoren hebben H2e functie en gedefinieerde hiaattijd vlgns BS h5x ? */
extern  void    Dt_rfsh(void);
extern  void    Dt_n_set(Int16,Mask16);                /* set nieuwe detector eigenschap    */
extern  void    Dt_n_reset(Int16,Mask16);              /* reset nieuwe detector eigenschap  */
extern  void    Dt_t_rset(Int16,Mask16,Bool);          /* set nieuwe detector type          */

extern  void    Dt_td_start(Mask16,Int16);             /* start tijd van detector           */
extern  void    Dt_td_herstart(Mask16,Int16);          /* herstart tijd van detector        */
extern  void    Dt_td_reset(Mask16,Int16);             /* reset tijd van detector           */
extern  void    Dt_td_haltr(Mask16,Int16);             /* halteer tijd van detector         */
extern  void    Dt_td_dehltr(Mask16,Int16);            /* dehalteer tijd van detector       */

extern  void    Dt_td_set_halt(Mask16,Int16);          /* halteer een tijd van detector     */
extern  void    Dt_td_set_door(Mask16,Int16);          /* start een tijd door van detector  */

extern  Uint32  Dt_td_waarde(Mask16,Int16);            /* lees tijdwaarde van detector uit  */
extern  Uint32  Dt_td_moment_waarde(Mask16,Int16);     /* lees momentele tijdwaarde van detector uit  */

extern  void    Dt_td_change(Mask16,Int16,Uint32);     /* verander tijdwaarde van detector samen met een "bijbehorende" tijd */
extern  void    Dt_td2_change(Mask16,Int16,Uint32);    /* verander tijdwaarde van detector  */

extern  void    Dt_kies_H(Int16);                      /* kies hiaattijd                    */
extern  void    Dt_kies_EH(Int16);                     /* kies extra hiaattijd              */

extern  Int16   Dt_num(void);

extern Int16    cif_iswijz_p_sec;

#endif  /* CRDTO_H */
