#ifndef CRHFO_H
#define CRHFO_H

/*
 * Copyright(c) 1992 RWS Dienst Verkeerkunde
 *
 * File         : crhfo.h
 *
 * Description  : functies en macro's hulpfunctie object
 *                Iedere functie die als macro gedefinierd kan worden
 *                wordt ook als macro gedefinieerd.
 *
 * History      : 08-03-90 Creation
 *                6-7-92,  aanpassing aan nieuwe interface definitie
 *                         van de commissie C van de CVN
 *
 */


#include "crhfo.d"
#include "crtypes.h"

#define         HF_MASK         1
#define         SCH_MASK        2
#define         TL_MASK         4
#define         TIJD_MASK       8
#define         KL_MASK        16
/* #define         EGGP_MASK      32 */
#define         REGSTAT_MASK   64

struct REGELING_STATUS
{
  Int16          nummer;
  Int16          debuggerNummer;
  Int16          status;
};

extern  struct UHF              uhf;
extern  struct TLF              tlf;
extern  Int16    nEparms;
extern  Int32    *pEparm_wd;
extern  Bool32K  eparm_tr;

extern  Int16    nEggparms;
extern  Int32    *pEggparm_wd;
extern  Bool32K  eggparm_tr;

extern  struct  REGELING_STATUS   REGELING_STATUS_tabel[ MAX_REGELINGEN ];

extern int CRHFO_ptp_wijziging;

/*
 * Support formules en macro's voor
 * het object signaalgroep
 *
 */

#define         Hf_val(a)       ( uhf.b[(a)]  & HF_MASK )
#define         Hf_begin(a)     ( uhf.bg[(a)] & HF_MASK )
#define         Hf_end(a)       ( uhf.en[(a)] & HF_MASK )
#define         Hf_tracon(a)    ( uhf.tr[(a)]|= HF_MASK )
#define         Hf_tracof(a)    ( uhf.tr[(a)]&= (Mask16)~HF_MASK )

#define         Kl_val(a)       ( uhf.b[(a)]  & KL_MASK )
#define         Kl_begin(a)     ( uhf.bg[(a)] & KL_MASK )
#define         Kl_end(a)       ( uhf.en[(a)] & KL_MASK )
#define         Kl_tracon(a)    ( uhf.tr[(a)]|= KL_MASK )
#define         Kl_tracof(a)    ( uhf.tr[(a)]&= (Mask16)~KL_MASK )

#define         Sch_val(a)      ( uhf.b[(a)]  & SCH_MASK )
#define         Sch_begin(a)    ( uhf.bg[(a)] & SCH_MASK )
#define         Sch_end(a)      ( uhf.en[(a)] & SCH_MASK )
#define         Sch_tracon(a)   ( uhf.tr[(a)]|= SCH_MASK )
#define         Sch_tracof(a)   ( uhf.tr[(a)]&= (Mask16)~SCH_MASK )

#define         EPARM_val(a)    ( pEparm_wd[(a)] )
#define         EPARM_tracon(a) ( eparm_tr[a/16]|= (Mask16)(1<<(a%16)) )
#define         EPARM_tracof(a) ( eparm_tr[a/16]&= (Mask16)~(1<<(a%16)) )

#define         EGGPARM_val(a)    ( pEggparm_wd[(a)] )
#define         EGGPARM_tracon(a) ( eggparm_tr[a/16]|= (Mask16)(1<<(a%16)) )
#define         EGGPARM_tracof(a) ( eggparm_tr[a/16]&= (Mask16)~(1<<(a%16)) )

#define         Tijd_val(a)     ( uhf.b[(a)]  & TIJD_MASK )
#define         Tijd_begin(a)   ( uhf.bg[(a)] & TIJD_MASK )
#define         Tijd_end(a)     ( uhf.en[(a)] & TIJD_MASK )
#define         Tijd_tracon(a)  ( uhf.tr[(a)]|= TIJD_MASK )
#define         Tijd_tracof(a)  ( uhf.tr[(a)]&= (Mask16)~TIJD_MASK )
#define         Tijd_wtracon(a) ( uhf.trw[(a)]|= TIJD_MASK )
#define         Tijd_wtracof(a) ( uhf.trw[(a)]&= (Mask16)~TIJD_MASK )
#define         Tijd_def(a)     ( uhf.p_td[(a)]!= NULL)    /* tijd gedefinieerd voor deze SG? */
#define         Tijd_halted(a)  ( uhf.hl[(a)] & TIJD_MASK )

#define         Tl_val(a)       ( uhf.teller[(a)])
#define         Tl_tracon(a)    ( uhf.tr[(a)]|= TL_MASK )
#define         Tl_tracof(a)    ( uhf.tr[(a)]&= (Mask16)~TL_MASK )

#define         Tlf_waarde(ix)  ( tlf.waarde[ix] )
#define         Tlf_tracon(ix)  ( tlf.volgnr |= ((Uint32) (1<<ix)))
#define         Tlf_tracof(ix)  ( tlf.volgnr &= (Uint32)~((Uint32) (1<<ix)))
#define         Tlf_set(ix,w)   ( tlf.waarde[ix]=w )

#define         Ovi_tracon(a)   ( ovi_tr[a/16]|= (Mask16)(1<<(a%16)) )
#define         Ovi_tracof(a)   ( ovi_tr[a/16]&= (Mask16)~(1<<(a%16)) )
#define         Ovu_tracon(a)   ( ovu_tr[a/16]|= (Mask16)(1<<(a%16)) )
#define         Ovu_tracof(a)   ( ovu_tr[a/16]&= (Mask16)~(1<<(a%16)) )

extern Uint32   REGSTAT_val(Uint16);            /* waarde van regelaarstatus */
extern void     REGSTAT_tracon(Uint16);         /* trace on van regelaarstatus */
extern void     REGSTAT_tracof(Uint16);         /* trace off van regelaarstatus */

extern  Bool    Hf_init(Int16);                 /* initieer de HF object */
extern  Bool    Tlf_init(void);                 /* initieer de tellersfunctie object */

extern  void    Hf_set(Int16);                  /* set hulpfunctie */
extern  void    Hf_reset(Int16);                /* reset hulpfunctie */

extern  void    Kl_set(Int16);                  /* set klok */
extern  void    Kl_reset(Int16);                /* reset klok */
extern  Int16   Kl_1e_waarde(Int16);            /* lees klok uit */
extern  Int16   Kl_2e_waarde(Int16);            /* lees klok uit */
extern  void    Kl_1e_change(Int16,Int16);      /* verander waarde van klok */
extern  void    Kl_2e_change(Int16,Int16);      /* verander waarde van klok */

extern  void    Sch_set(Int16);                 /* set schakelaar  */
extern  void    Sch_reset(Int16);               /* reset schakelaar */

extern  void    EGGPARM_change(Int16, Int32);   /* verander waarde van extra geheel getal parameter */
extern  void    EPARM_change(Int16, Int32);     /* verander waarde van extra parameter */

extern  void    Tijd_start(Int16);              /* start tijd */
extern  void    Tijd_herstart(Int16);           /* herstart tijd */
extern  void    Tijd_reset(Int16);              /* reset tijd */
extern  void    Tijd_haltr(Int16);              /* halteer tijd */
extern  void    Tijd_dehltr(Int16);             /* dehalteer tijd */
extern  void    Tijd_change(Int16,Uint32);      /* verander tijd */
extern  Uint32  Tijd_waarde(Int16);             /* lees tijdwaarde van signaal groep uit  */
extern  Uint32  Tijd_moment_waarde(Int16);      /* lees momentele tijdwaarde van signaal groep uit  */

extern  void    Tl_incr(Int16);                 /* increment teller */
extern  void    Tl_decr(Int16);                 /* decrement teller */
extern  void    Tl_set(Int16,Uint16);            /* set teller */

extern  void    Klok_eval(void);                /* evalueer klokken */

extern  void    Hf_rfsh(void);                  /* refresh old-waardes */

extern  void    set_datum(Uint32);              /* set systeemdatum */
extern  void    set_tijd(Uint32);               /* set systeemtijd */

extern  Bool    Kp_init(Int16 *);               /* initieer de koppelinstructies */
extern  void    Kp_rset(Int16, Bool);           /* toewijzingsinstructies */
extern  void    Kp_input_ptp_update(Int16 *);   /* ververs PTP ingangssignalen */
extern  void    Kp_input(Int16 *);              /* lees koppelsignalen in */
extern  Bool    Kp_output(Int16 *);             /* stuur koppelsignalen uit */
extern  void    Kp_reset_input(Int16);          /* reset koppelsignalen     */

extern  void    regeling_init(void);            /* Initialisatie REGELING_STATUS_tabel */
extern  void    regeling_update(void);          /* Update REGELING_STATUS_tabel */
extern  Int16   status_regelen(Int16);          /* Bepalen of bepaalde regeling aan het regelen is */
extern  Bool    status_PTP(Int16);              /* Bepalen of PTP-koppeling met bepaalde regeling intact is */

#endif  /* CRHFO_H */
