#ifndef CRTYPES_H
#define CRTYPES_H

/*
 * Copyright(c) 1992 RWS Dienst Verkeerkunde
 *
 * File         : crtypes.h
 *
 * Description  : Gemeenschappelijke macro-definities.
 *
 */

#include "rws_dim.h"

/***************************************************************************/
/*  B R E N G    H I E R    V E R A N D E R I N G E N  A A N   O M   D I T */
/*  P A K K E T   A A N  U W   W E N S E N   A A N  T E   P A S S E N ! !  */
/*  de systeemgrenzen liggen vast in de file rws_dim.h                     */
/***************************************************************************/

/*------- systeem-grenzen -------*/

#define         MAX_SG          CIF_AANT_US_FC    /* maximum aantal SG's     */
#define         MAX_GRSG        ((MAX_SG+15)/16)  /* max. # groepen met SG's */
#define         MAX_DET         CIF_AANT_IS_D     /* maximum aantal DET's    */
#define         MAX_GRDET       ((MAX_DET+15)/16) /* max. # groepen met DET's*/
#define         MAX_BK          RWS_AANT_BK       /* maximum aantal blokken  */
#define         MAX_RGA         RWS_AANT_RGA      /* maximum aantal rga's    */

/*------- elementaire datatypes -------*/
typedef         unsigned short  Mask16;           /*   16-bits bitmasker     */
typedef         unsigned long   Mask32;           /*   32-bits bitmasker     */
typedef         unsigned short  Bool;             /*      boolean            */
typedef         unsigned short  Bool16;           /*   16 booleans           */
typedef         unsigned long   Bool32;           /*   32 booleans           */
typedef         unsigned short  Bool64[4];        /*   64 booleans           */
typedef         unsigned short  Bool128[8];       /*  128 booleans           */
typedef         unsigned short  Bool4096[256];    /* 4096 booleans           */
typedef         unsigned short  Bool32K[2048];   /* 32768 booleans           */
typedef         char            Int8;             /*    8 bits integer       */
typedef         short           Int16;            /*   16 bits integer       */
typedef         long            Int32;            /*   32 bits integer       */
typedef         unsigned char   Uint8;            /*    8 bits unsigned int  */
typedef         unsigned short  Uint16;           /*   16 bits unsigned int  */
typedef         unsigned long   Uint32;           /*   32 bits unsigned int  */
typedef         Bool            (*Ofunc)(void);   /*   pointer naar functie  */
typedef         void            (*Aktie)(Int16,Int16);
                                                  /*   pointer naar functie  */
typedef         double          Float;            /*   floating point waarde */
typedef         char            *Tekst;           /*   tekststring           */
typedef         short           Istr64[64];       /*   array van 64 integers */

/***************************************************************************/
/*  B R E N G    H I E R O N D E R   G E E N   V E R A N D E R I N G E N   */
/*                             A A N !                                     */
/***************************************************************************/

#define         TRUE            1               /* logical TRUE */
#define         FALSE           0               /* logical FALSE */
#define         LEEG            -1              /* einde id. van een string */


#ifndef NULL
#define NULL 0
#endif

#ifndef max
#define max(a,b)            (((a) > (b)) ? (a) : (b))
#endif

#define         Int8_max        ((Int8)(0x7f))
#define         Uint8_max       ((Uint8)(0xff))
#define         Int16_max       ((Int16)(0x7fff))
#define         Uint16_max      ((Uint16)(0xffff))
#define         Int32_max       ((Int32)(0x7fffffff))
#define         Uint32_max      ((Uint32)(0xffffffffL))

/* signaalgroep */
/* ------------ */
#define SGT_VRI         0                   /* Default verkeerslantaarn */
#define SGT_MTM         1                   /* MTM ult verkeerslantaarn */

/*-------  Hulpfunctie-macro's, eenmalig -------*/
#define         __HF_EPARM        0
#define         __HF_EGGPARM      1
#define         __HF_TIJD         2
#define         __HF_TELLER       3
#define         __HF_OVI          4
#define         __HF_OVU          5

/*-------  Hulpfunctie-macro's, SG gekoppeld -------*/
#define         __A               0
#define         __AFK             1
#define         __BL              2
#define         __D               3
#define         __DA              4
#define         __DK              5
#define         __MA              6
#define         __PPV             7
#define         __TR              8
#define         __VAA             9
#define         __VNM             10
#define         __BS_Z            11
#define         __BS_BC           12
#define         __BS_BG           13
#define         __RWR             14
#define         __SR              15
#define         __PPB             16
#define         __PPAH            17
#define         __VNMH            18
#define         __WSRH            19
#define         __BR              20
#define         __PPS             21
#define         __TO_PRIM         22
#define         __TMVG            23
#define         __VRVG            24
#define         __VMVG            25
#define         __VVAG2e          26
#define         __RVAG2e          27
#define         __BA              28
#define         MAX_HFS           29

/* special teneinde de status van de beurt door te geven aan de debugger */
/* dient dezelfde waarde te hebben als MAX_HFS                           */
/* als MAX_HFS veranderd, dan ook de MACRODEF.AVV file hierop aanpassen  */
#define         __BRT             MAX_HFS

/*------- Hulpfunctie-macro's, BK gekoppeld -------*/

#define         __BLOK            0
#define         __BS_CF           1     /* hfdst. 6.2 */
#define         MAX_HFB           2

/*------- Hulpfunctie-macro's, SG & BK gekoppeld -------*/

#define         __AR              0
#define         __PG              1
#define         __PPA             2
#define         __PPP             3
#define         __PR              4
#define         __MAH4            5     /* in wezen SG en volgnr. gekoppeld */
#define         __MAH5            6     /* in wezen SG en volgnr. gekoppeld */
#define         __MAH6            7     /* in wezen SG en volgnr. gekoppeld */
#define         __MAH7            8     /* in wezen SG en volgnr. gekoppeld */
#define         __RGAH            9     /* in wezen SG en volgnr. gekoppeld */
#define        MAX_HFBS          10

/*------- Optie-macro's, SG gekoppeld -------*/
#define         __AFK_OP          0
#define         __BL_OP           1
#define         __MA_OP           2
#define         __TMVG_OP         3
#define         __PPV_OP          4
#define         __TR_OP           5
#define         __VAA_OP          6
#define         __VRVG_OP         7
#define         __VVAG2e_OP       8
#define         __VMVG_OP         9
#define         __PPB_OP          10
#define         __RWG_OP          11
#define         __RWR_OP          12
#define         __RVAG2e_OP       13
#define         __PPS_OP          14
#define         MAX_OPS           15

/*------- Optie-macro's, SG & BK gekoppeld -------*/
#define         __PPA_OP          0
#define         __PPP_OP          1
#define         __VNM_OP          2
#define         MAX_OPBS          3

/*------- Optie-macro's, BK gekoppeld -------*/
#define         __BO_OP           0
#define         MAX_OPB           1

/*------- Toestand-macro's SG gekoppeld -------*/
#define         __WR              0
#define         __ROG             1
#define         __RVG             2
#define         __VG              3
#define         __VAG_1e          4
#define         __WG              5
#define         __VAG_2e          6
#define         __MVG             7
#define         __R               8
#define         __G               9
#define         __GL              10
#define         MAX_TS            11

/*------- Tijd-macro's, SG gekoppeld -------*/
#define         __VG_TIJD         0
/* 1 is beschikbaar voor de toekomst */
/* 2 is beschikbaar voor de toekomst */
#define         __GG_TIJD         3
#define         __GR_TIJD         4
#define         __GL_TIJD         5
#define         __MG_TIJD         6
#define         __GGL_TIJD        7
/* 8 is beschikbaar voor de toekomst */
/* 9 is beschikbaar voor de toekomst */
#define         __MG_TIJD1        11
#define         __MG_TIJD2        12
#define         __MG_TIJD3        13
#define         __MG_TIJD4        14
#define         __MG_TIJD5        15
#define         __MG_TIJD6        16
/* 17 is beschikbaar voor de toekomst */
/* 18 is beschikbaar voor de toekomst */
#define         __MG_TIJD7        19
#define         __MG_TIJD8        20
#define         __MG_TIJD9        21
#define         __MG_TIJD10       22
#define         __MG_TIJD11       23
#define         __MG_TIJD12       24
#define         MAX_TD            25

/* teneinde de interne signaalgroeptoestand door te geven aan          */
/* de debugger: dient dezelfde waarde te hebben als MAX_TD             */
/* als MAX_TD veranderd, dan ook de MACRODEF.AVV file hierop aanpassen */
#define         __INT_TOESTAND    MAX_TD

/*------- Tijd-macro's, SG gekoppeld -------*/
#define         __O_TIJD          0
#define         __GO_TIJD         1
#define         __INT             2
#define         MAX_O_TD          3

/*------- Tijd-macro's, Detector gekoppeld -------*/
#define         __BZT_TIJD        0
#define         __H_TIJD          1
#define         __H_TIJD1         2
#define         __H_TIJD2         3
#define         MAX_D_TD          4

/*------- User defined Items ---------------*/
#define         __HF              0
#define         __KLOK            1
#define         __KLOK_1e         2
#define         __KLOK_2e         3
#define         __SCH             4
#define         __TIJD            5
/*  6 is beschikbaar voor de toekomst */
#define         __DFOUT           7
#define         __DFOUT_BG        8
#define         __DFOUT_OG        9
#define         __DWD            10
#define         __DFOUT_FG       11
/* 12 is beschikbaar voor de toekomst */
#define         __REGSTAT        13
#define         __DVWD           14
/* 15 is gereserveerd voor __BZT_TIJD + 15 */
/* 16 is gereserveerd voor __H_TIJD   + 15 */
/* 17 is gereserveerd voor __H_TIJD1  + 15 */
/* 18 is gereserveerd voor __H_TIJD2  + 15 */
#define         MAX_HF           256

/*------- tellers -------*/
#define         __RONDE_TELLER    0
#define         __CR_DATUM        1
#define         __CR_TIJD         2
#define         __CR_DAGSOORT     3
#define         __CR_ERROR        4
#define         __CR_ERR1         5
#define         __CR_ERR2         6
#define         __CR_ERR3         7
#define         __CR_VERSIE       8
#define         __CR_DAGNUMMER    9
#define         __CR_MAANDNUMMER 10
#define         __CR_JAARNUMMER  11

/*------- datastructures -------*/

typedef struct
{
         Int16   jaar,
                 maand,
                 dag,
                 dagsoort,
                 uur,
                 minuut,
                 seconde,
                 sec_teller,
                 tiende_sec_teller;
} Klok;


/*------- macro-functions -------*/

#define set_bit_Bool64(set,sg)   (set[sg/16]|=(Mask16)(1<<(sg%16)))
#define in_sg_set(set,sg)        (set[sg/16]&(1<<(sg%16)))

#if  (MAX_GRSG == 4)
  #define clear_Bool64(b)        (b[0]=b[1]=b[2]=b[3]=0)
  #define val_Bool64(b1)         (b1[0]||b1[1]||b1[2]||b1[3])
  #define and_Bool64(b1,b2)      ((b1[0]&b2[0])||(b1[1]&b2[1])||(b1[2]&b2[2])||(b1[3]&b2[3]))
  #define or_Bool64(b1,b2)       ((b1[0]|b2[0])||(b1[1]|b2[1])||(b1[2]|b2[2])||(b1[3]|b2[3]))
  #define cmp_Bool64(b1,b2)      ((b1[0]==b2[0])&&(b1[1]==b2[1])&&(b1[2]==b2[2])&&(b1[3]==b2[3]))
  #define or_add_Bool64(b1,b2)   (b1[0]|=b2[0],b1[1]|=b2[1],b1[2]|=b2[2],b1[3]|=b2[3])
  #define and_add_Bool64(b1,b2)  (b1[0]&=b2[0],b1[1]&=b2[1],b1[2]&=b2[2],b1[3]&=b2[3])
  #define inv_Bool64(b1)         (b1[0]=(Mask16)~(unsigned)b1[0],b1[1]=(Mask16)~(unsigned)b1[1],b1[2]=(Mask16)~(unsigned)b1[2],b1[3]=(Mask16)~(unsigned)b1[3])

#elif  (MAX_GRSG == 3)
  #define clear_Bool64(b)        (b[0]=b[1]=b[2]=0)
  #define val_Bool64(b1)         (b1[0]||b1[1]||b1[2])
  #define and_Bool64(b1,b2)      ((b1[0]&b2[0])||(b1[1]&b2[1])||(b1[2]&b2[2]))
  #define or_Bool64(b1,b2)       ((b1[0]|b2[0])||(b1[1]|b2[1])||(b1[2]|b2[2]))
  #define cmp_Bool64(b1,b2)      ((b1[0]==b2[0])&&(b1[1]==b2[1])&&(b1[2]==b2[2]))
  #define or_add_Bool64(b1,b2)   (b1[0]|=b2[0],b1[1]|=b2[1],b1[2]|=b2[2])
  #define and_add_Bool64(b1,b2)  (b1[0]&=b2[0],b1[1]&=b2[1],b1[2]&=b2[2])
  #define inv_Bool64(b1)         (b1[0]=(Mask16)~(unsigned)b1[0],b1[1]=(Mask16)~(unsigned)b1[1],b1[2]=(Mask16)~(unsigned)b1[2])

#elif  (MAX_GRSG == 2)
  #define clear_Bool64(b)        (b[0]=b[1]=0)
  #define val_Bool64(b1)         (b1[0]||b1[1])
  #define and_Bool64(b1,b2)      ((b1[0]&b2[0])||(b1[1]&b2[1]))
  #define or_Bool64(b1,b2)       ((b1[0]|b2[0])||(b1[1]|b2[1]))
  #define cmp_Bool64(b1,b2)      ((b1[0]==b2[0])&&(b1[1]==b2[1]))
  #define or_add_Bool64(b1,b2)   (b1[0]|=b2[0],b1[1]|=b2[1])
  #define and_add_Bool64(b1,b2)  (b1[0]&=b2[0],b1[1]&=b2[1])
  #define inv_Bool64(b1)         (b1[0]=(Mask16)~(unsigned)b1[0],b1[1]=(Mask16)~(unsigned)b1[1])

#elif  (MAX_GRSG == 1)
  #define clear_Bool64(b)        (b[0]=0)
  #define val_Bool64(b1)         (b1[0])
  #define and_Bool64(b1,b2)      (b1[0]&b2[0])
  #define or_Bool64(b1,b2)       (b1[0]|b2[0])
  #define cmp_Bool64(b1,b2)      (b1[0]==b2[0])
  #define or_add_Bool64(b1,b2)   (b1[0]|=b2[0])
  #define and_add_Bool64(b1,b2)  (b1[0]&=b2[0])
  #define inv_Bool64(b1)         (b1[0]=(Mask16)~(unsigned)b1[0])

#endif

/*------- global declarations -------*/
extern  Bool                     BLOKKENSCHEMA_ACTIEF; /* name says all...          */
extern  Bool                     ALLES_ROOD_APPLICATIE; /* name says all...         */
extern  Bool                     busy_going_to_all_red; /* name says all...         */
extern  Int16                    sgn_n;          /* signaalgroep nummer             */
extern  Int16                    sgn_k;
extern  Int16                    sgn_max;
extern  Int16                    num_bk;         /* aantal blokken                  */
extern  Int16                    blk_m;          /* blok-nr van 't actieve blok     */
extern  Int16                    blk_mb;
extern  Int16                    blk_l;          /* blok-nr                         */
extern  Int16                    det_max;        /* aantal detectoren in deze regeling            */
extern  Int32                    t_delta;        /* verschil in 1/10 sec tussen de vorige aanroep */
extern  Int16                    fix_bit;        /* fixatie bit                                   */
extern  Int16                    nr_test_koppelsignaal; /* nummer tbv testen koppelsignalen       */
extern  Bool128                  ovi_tr;
extern  Bool128                  ovu_tr;
extern  Int16                    ovi_huidige_stand[];


/*------- prototypes -------*/
extern  void                     verwerk_inkomende_berichten(void);
extern  void                     verwerk_koppeling(void);
extern  void                     afhandeling_set_BA(void);
extern  void                     afhandeling_set_DA(void);
extern  Bool                     afhandeling_set_MA_A(void);
extern  void                     reset_afhandeling_DA_BA_MA_A(void);
extern  void                     diverse_opties_H5_4(void);
extern  void                     overige_voorwaarden(void);
extern  void                     signaalgroep_procedure(Int16);
extern  void                     blok_procedure(void);
extern  void                     set_fb (void);
extern  void                     reset_fb (void);
extern  void                     set_wb (void);
extern  void                     reset_wb (void);
extern  void                     wacht_op_WUS(Int16);
extern  void                     dump_trigger(void);
extern  void                     plaats_in_interface (Int16, Int16);
extern  void                     setMtmStatus(Int16,Int16);
extern  Int16                    getMtmStatus(Int16);
extern  void                     ovi_lees_in( void );
extern  void                     set_reden_extra_wachttijd(Int16, Mask16);
extern  void                     reset_reden_extra_wachttijd(Int16, Mask16);
extern  void                     reset_alle_redenen_extra_wachttijd(Int16);
extern  void                     set_sg_timing(Int16, Int16, Int16, Int16, Int16, Int16, Int16, Int16);
extern  void                     reset_sg_timings(void);

/*----- wijzigingen blokkenschema -----*/
extern  Int16                    bepaal_num_bk( void );
extern  void                     reset_BLOKKENSCHEMA( void );
extern  Bool                     set_INITIEEL_BLOKKENSCHEMA( void );
extern  void                     maak_ALLES_ROOD( void );
extern  void                     set_PRIM_toewijzing( Int16 sgn, Int16 bkl );
extern  void                     reset_PRIM_toewijzing( Int16 sgn, Int16 bkl );
extern  void                     set_ALT_toewijzing( Int16 sgn, Int16 bkl );
extern  void                     reset_ALT_toewijzing( Int16 sgn, Int16 bkl );

/*----- PTP koppeling -----*/
#define CRHFO_PTP_PORT           1

/*----- CIF_FC_TIMING -----*/
#define START_ONBEKEND      -32768  /* starttijd onbekend */
#define GW                      -1  /* Geen Waarde (geen tijd of tijd onbekend) */

#endif  /* CRTYPES_H */
