#ifndef CRVLOG_H
#define CRVLOG_H

/*
 * Copyright(c) 2012 Rijkswaterstaat Dienst Verkeer en Scheepvaart
 *
 * File         : crvlog.h
 *
 * Description  : VLOG definities
 *
 */

/*------- defines -------*/
#define VLOG_VERSIE_MAJOR               3     /* huidige ondersteunde versie van VLOG protocol */
#define VLOG_VERSIE_MINOR               0     /* huidige ondersteunde versie van VLOG protocol */
#define VLOG_VERSIE_PATCH               0     /* huidige ondersteunde versie van VLOG protocol */

#define VLOG_AANT_GPS                   2     /* conform VLOG                                  */
#define VLOG_AANT_WPS                   3     /* conform VLOG                                  */

/*------- VLOG berichttypen -------*/
#define BERICHTTYPE_TIJD                1     /* tijd referentie bericht                       */
#define BERICHTTYPE_VLOGINFO            4     /* V-LOG informatie bericht                      */
#define BERICHTTYPE_DETECTIE            5     /* detectie informatie                           */
#define BERICHTTYPE_OV_ING              7     /* overige ingangen                              */
#define BERICHTTYPE_GUS_FC              9     /* interne fasecyclus/aanvraag/realisatiewijze   */
#define BERICHTTYPE_GUS_OV_UITG        11     /* overige uitgangen (GUS)                       */
#define BERICHTTYPE_WUS_SG             13     /* externe signaalgroepstatus (WUS)              */
#define BERICHTTYPE_WUS_OV_UITG        15     /* overige uitgangen (WUS)                       */
#define BERICHTTYPE_GPS                17     /* gewenste programma status (GPS)               */
#define BERICHTTYPE_WPS                19     /* werkelijke programma status (WPS)             */
#define BERICHTTYPE_INSTRUCTIE         31     /* status instructievariabelen                   */
#define BERICHTTYPE_FC_RWT             37     /* reden extra wachttijd                         */
#define BERICHTTYPE_OMGEVING           39     /* omgevingsfactoren                             */
#define BERICHTTYPE_OV_ING_MULTIVALENT 53      /* overige ingangen multivalent (incl num_ovi)        */
#define BERICHTTYPE_GUS_OV_UITG_MULTIVALENT 55 /* overige GUS uitgangen multivalent (incl num_ovu)   */
#define BERICHTTYPE_WUS_OV_UITG_MULTIVALENT 57 /* overige WUS uitgangen multivalent (incl num_ovu)   */
#define BERICHTTYPE_ACTUELE_MODULE          59 /* C-regelaar: bloknr                                 */
#define BERICHTTYPE_CONFIGURATIE           125 /* configuratiebericht                                */
#define BERICHTTYPE_CONTROLEBERICHT        127 /* controlebericht                                    */

#define BERICHTTYPE_WIJZ_TIJDCORRECTIE  0     /* tijd correctie bericht                                    */
#define BERICHTTYPE_WIJZ_DETECTIE       6     /* wijzigingen detectie informatie                           */
#define BERICHTTYPE_WIJZ_OV_ING         8     /* wijzigingen overige ingangen                              */
#define BERICHTTYPE_WIJZ_GUS_FC        10     /* wijzigingen interne fasecyclus/aanvraag/realisatiewijze   */
#define BERICHTTYPE_WIJZ_GUS_OV_UITG   12     /* wijzigingen overige uitgangen (GUS)                       */
#define BERICHTTYPE_WIJZ_WUS_SG        14     /* wijzigingen externe signaalgroepstatus (WUS)              */
#define BERICHTTYPE_WIJZ_WUS_OV_UITG   16     /* wijzigingen overige uitgangen (WUS)                       */
#define BERICHTTYPE_WIJZ_GPS           18     /* wijzigingen gewenste programma status (GPS)               */
#define BERICHTTYPE_WIJZ_WPS           20     /* wijzigingen werkelijke programma status (WPS)             */
#define BERICHTTYPE_WIJZ_DETSPEED      26     /* wijzigingen snelheidsdetector berichten                   */
#define BERICHTTYPE_WIJZ_KAR           28     /* wijzigingen selectieve detectie informatie (KAR)          */
#define BERICHTTYPE_WIJZ_INSTRUCTIE    32     /* wijzigingen status instructievariabelen                   */
#define BERICHTTYPE_WIJZ_FC_TIMING     36     /* wijzigingen fasecyclus timing                             */
#define BERICHTTYPE_WIJZ_FC_RWT        38     /* wijzigingen reden extra wachttijd                         */
#define BERICHTTYPE_WIJZ_OMGEVING      40     /* wijziging omgevingsfactoren                               */
#define BERICHTTYPE_WIJZ_OV_ING_MULTIVALENT 54      /* wijzigingen overige ingangen multivalent (incl num_ovi)        */
#define BERICHTTYPE_WIJZ_GUS_OV_UITG_MULTIVALENT 56 /* wijzigingen overige GUS uitgangen multivalent (incl num_ovu)   */
#define BERICHTTYPE_WIJZ_WUS_OV_UITG_MULTIVALENT 58 /* wijzigingen overige WUS uitgangen multivalent (incl num_ovu)   */
#define BERICHTTYPE_WIJZ_ACTUELE_MODULE          60 /* C-regelaar: bloknr                                  */
#define BERICHTTYPE_WIJZ_DETLENGTH               62 /* wijzigingen lengtedetectie berichten                */
#define BERICHTTYPE_REALTIME_CONTROLEBERICHT    128 /* realtime controlebericht                            */

/*------- VLOGMODES -------*/
#define VLOGMODE_NONE                   0     /* niks doen                                     */
#define VLOGMODE_REALTIME_BINAIR        1     /* realtime stream, binair formaat               */
#define VLOGMODE_FILE_BINAIR            2     /* naar file, binair formaat                     */
#define VLOGMODE_REALTIME_ASCII         3     /* realtime stream, ASCII formaat                */
#define VLOGMODE_FILE_ASCII             4     /* naar file, ASCII formaat                      */
#define VLOGMODE_MAX                    5     /* bovengrens                                    */

/*------- Interne status fasecyclus -------*/
#define FC_ROG                       0x10     /* recht op groen         */
#define FC_RVG                       0x11     /* rood voor groen        */
#define FC_VG                        0x12     /* vast groen             */
#define FC_VAG_1e                    0x13     /* va groen               */
#define FC_WG                        0x14     /* wachtgroen             */
#define FC_VAG_2e                    0x15     /* va groen               */
#define FC_MVG                       0x16     /* meeverlenggroen        */
#define FC_GL                        0x17     /* geel                   */
#define FC_WR                        0x18     /* wachtrood              */

#define SYN                          0x16     /* synchronisatie - einde bericht                 */
#define EOL                          '\n'     /* synchronisatie - einde bericht ASCII = newline */

/* typen VLOG detectoren en ingangssignalen */
#define VLOG_DL_TYPE               0x0001
#define VLOG_DK_TYPE               0x0002
#define VLOG_DSI_TYPE              0x0004
#define VLOG_ISV_TYPE              0x0008
#define VLOG_ISL_TYPE              0x0010
#define VLOG_ISM_TYPE              0x0080
#define VLOG_KOP_TYPE              0x0100
#define VLOG_LNG_TYPE              0x0200
#define VLOG_VER_TYPE              0x0400
#define VLOG_VOOR_TYPE             0x0800
#define VLOG_DKOP_TYPE             0x0101
#define VLOG_DLNG_TYPE             0x0201
#define VLOG_DVER_TYPE             0x0401
#define VLOG_DSUIT_TYPE            0x0104
#define VLOG_DSIN_TYPE             0x0404
#define VLOG_DSVOOR_TYPE           0x0804

/* typen VLOG signaalgroepen en uitgangssignalen */
#define VLOG_MVT_TYPE              0x0001
#define VLOG_VTG_TYPE              0x0002
#define VLOG_FTS_TYPE              0x0004
#define VLOG_OV_TYPE               0x0008
#define VLOG_USM_TYPE              0x0080


/*------- prototypes -------*/
extern void setVlogMode(char mode);           /* functie-aanroep door eindgebruiker            */
extern void setVlogMode2(char mode);          /* functie-aanroep door eindgebruiker            */
extern void setVlogVRIid(char *s);            /* functie-aanroep door eindgebruiker            */

extern void schrijfVlogData(void);            /* functie-aanroep door C-regelaar               */

extern Bool schrijfBericht(unsigned char c);
extern void schrijfEindeBericht(void);
extern Bool schrijf_FILE_UBER(unsigned char c);

extern void schrijfTijdCorrectieBericht(void);
extern void schrijfTijdReferentieBericht(void);
extern void schrijfVLogInformatieBericht(void);

extern void schrijfStatusBerichtDETECTIE(void);
extern void schrijfStatusBerichtOV_ING(void);
extern void schrijfStatusBerichtGUS_FC(void);
extern void schrijfStatusBerichtGUS_OV_UITG(void);
extern void schrijfStatusBerichtWUS_SG(void);
extern void schrijfStatusBerichtWUS_OV_UITG(void);
extern void schrijfStatusBerichtGPS(void);
extern void schrijfStatusBerichtWPS(void);
extern void schrijfStatusBerichtInstructies(void);
extern void schrijfStatusBerichtRWT(void);
extern void schrijfStatusBerichtOmgeving(void);
extern void schrijfStatusBerichtOV_ING_Multi(void);
extern void schrijfStatusBerichtGUS_OV_UITG_Multi(void);
extern void schrijfStatusBerichtWUS_OV_UITG_Multi(void);
extern void schrijfStatusBerichtActueleModule(void);
extern void schrijfConfiguratieBericht(void);
extern void schrijfControleBericht(void);

extern void schrijfWijzigingsBerichtDETECTIE(void);
extern void schrijfWijzigingsBerichtOV_ING(void);
extern void schrijfWijzigingsBerichtGUS_FC(void);
extern void schrijfWijzigingsBerichtGUS_OV_UITG(void);
extern void schrijfWijzigingsBerichtWUS_SG(void);
extern void schrijfWijzigingsBerichtWUS_OV_UITG(void);
extern void schrijfWijzigingsBerichtGPS(void);
extern void schrijfWijzigingsBerichtWPS(void);
extern void schrijfWijzigingsBerichtInstructies(void);
extern void schrijfWijzigingsBerichtDetSpeed(void);
extern void schrijfWijzigingsBerichtKAR(void);
extern void schrijfWijzigingsBerichtTiming(void);
extern void schrijfWijzigingsBerichtRWT(void);
extern void schrijfWijzigingsBerichtOmgeving(void);
extern void schrijfWijzigingsBerichtOV_ING_Multi(void);
extern void schrijfWijzigingsBerichtGUS_OV_UITG_Multi(void);
extern void schrijfWijzigingsBerichtWUS_OV_UITG_Multi(void);
extern void schrijfWijzigingsBerichtActueleModule(void);
extern void schrijfWijzigingsBerichtDetLength(void);
extern void schrijfRealtimeControleBericht(void);

extern Bool ToevoegenAanCfgFile(char *text);
extern void MaakVLogCfgFileInMem(void);
extern void updateTijd(void);
extern void bewaarTijd(void);

struct VLOG_CFG
{
  Int16          nummer;
  char*          naam;
  Int16          type;
};

extern struct   VLOG_CFG          VLOG_SG_tabel[];   /* namen signaalgroepen */
extern struct   VLOG_CFG          VLOG_DET_tabel[];  /* namen detectoren */
extern struct   VLOG_CFG          VLOG_IKS_tabel[];  /* namen inkomende koppelsignalen */
extern struct   VLOG_CFG          VLOG_OVI_tabel[];  /* namen overige ingangssignalen */
extern struct   VLOG_CFG          VLOG_UKS_tabel[];  /* namen uitgaande koppelsignalen */
extern struct   VLOG_CFG          VLOG_OVU_tabel[];  /* namen overige uitgangssignalen */

extern int vlog_n_sg;
extern int vlog_n_det;
extern int vlog_n_iks;
extern int vlog_n_ovi;
extern int vlog_n_uks;
extern int vlog_n_ovu;


#endif  /* CRVLOG_H */
