/*************************************************************************
*  Program Name  : FLASH (FLexit Application SHell)                      *
*========================================================================*
*                                                                        *
*  Filename      : evtdisp.h                                             *
*                                                                        *
*  Function      : External header file for the event dispatcher.        *
*                                                                        *
*  Date Created  : December 22, 1995                                     *
*                                                                        *
*  Version       : 1.0                                                   *
*                                                                        *
*  Author(s)     : E.E. de Vries (Ed)                                    *
*                                                                        *
*  Files used    :                                                       *
*                                                                        *
*  Date Modified : See RCS identification                                *
*                                                                        *
*  Modifications :                                                       *
*                                                                        *
*  Comments      :                                                       *
*                                                                        *
*------------------------------------------------------------------------*
*  COPYRIGHT (c) 1995 by A.N.D. Identification B.V. Rotterdam.           *
*************************************************************************/

#ifdef __cplusplus
extern "C" {
#endif


/*----------------------------------------------------------------------*/
/* CONSTANT DEFINITIONS                                                 */
/*----------------------------------------------------------------------*/

/* operation modes */
/* --------------- */
#define ED_UNARY_TYPE   long               /* The event data unary type */

/* operation modes */
/* --------------- */
#define ST_STOP          0                           /* The clock stops */
#define ST_CONTINUE      1                       /* The clock continues */
#define ST_STEP          2      /* The clock takes one granularity step */

/* time constants */
/* -------------- */
#define ST_MAXDIFF   0x20000000L /* Max. difference between time values */
#define ST_MAX       0x40000000L              /* Max. timer ticks value */
#define ST_NOW       0x40000000L                 /* Send event now time */
#define ST_NEXT(X)   st_get((long)(X)) /* Send next event after X msec. */

/* limit values */
/* ------------ */
#define MUL_MIN            1L               /* Min. multiply percentage */
#define MUL_MAX        10000L               /* Max. multiply percentage */

/* error values */
/* ------------ */
#define ERR_NOMSG        (-1)                       /* No more messages */
#define ERR_NOHDL        (-2)              /* No handler for this event */
#define ERR_OVFHDL       (-3)                /* Too many event handlers */
#define ERR_DUPHDL       (-4)           /* Event handler already exists */
#define ERR_NOMEM        (-5)                          /* Out of memory */
#define ERR_EMPTY        (-6)                    /* Emtpy message queue */
#define ERR_NOWND        (-7)                /* Could not create window */

/* message modes */
/* ------------- */
#define MM_NORMAL       0         /* Normal (timed) event dispatch mode */
#define MM_TURBO        1    /* Turbo (do not wait) event dispatch mode */

/* detector types */
/* -------------- */
#define DT_UNKNOWN      0                 /* Unknown or don't care type */
#define DT_SELECTIVE    1                         /* Selective detector */
#define DT_SPEED        2                             /* Speed detector */
#define DT_PRESENCE     3                          /* Presence detector */
#define DT_ANALOG       4                            /* Analog detector */
#define DT_SHORTLOOP    5                        /* Short loop detector */

/* signaalgroep */
/* ------------ */
#define SGT_VRI         0                   /* Default verkeerslantaarn */
#define SGT_MTM         1                   /* MTM ult verkeerslantaarn */

/* module identification numbers */
/* ----------------------------- */
#define MID_NULL              0            /* ID of none of the modules */
#define MID_CREG(X)         (X)           /* ID of kruispuntsregeling X */
#define MID_FLXSIM         1001               /* ID of simulator module */
#define MID_FLXRGL         1002         /* ID of FLXCOL regelaar module */
#define MID_WORKBENCH      2000               /* ID of workbench module */
#define MID_CLOCK          2001           /* ID of simulated time clock */
#define MID_MONITOR        2002                 /* ID of monitor module */
#define MID_CRDEBUG(X)   (3000+X)   /* ID of C-Regelaar debugger module */


/* event codes algemeen */
/* -------------------- */
#define EC_REVIVAL            0                              /* Revival */


/* event codes messages */
/* -------------------- */
#define EC_MSG_TEXT         100             /* Toon een tekst boodschap */
#define EC_MSG_WARNING      101                /* Toon een waarschuwing */
#define EC_MSG_ERROR        102                 /* Toon een foutmelding */
#define EC_MSG_FATAL        103         /* Toon een fatale fout melding */


/* event codes evtdisp(+timer) */
/* --------------------------- */
#define EC_REALTIME         200              /* The 'real' time changed */
#define EC_MAXTIME          201    /* Time exceeds max. simulation time */


/* event codes flxsim */
/* ------------------ */
#define EC_FLXSIM_INIT         1000                     /* Initialiseer */
#define EC_FLXSIM_NEW          1001                   /* Init new start */
#define EC_FLXSIM_RESUME       1002               /* Init resumed start */
#define EC_FLXSIM_START        1003                     /* Do start, go */
#define EC_FLXSIM_STOP         1004                             /* Stop */
#define EC_FLXSIM_STOP_AFTER   1005     /* Stop after simulation subrun */
#define EC_FLXSIM_TRACE        1006     /* FLXSIM sends trace to FLXRGL */
#define EC_FLXSIM_COMPLETED    1009          /* Simulation is completed */

#define EC_FLXSIM_ACK_INIT     1010                 /* Acknowledge init */
#define EC_FLXSIM_ACK_NEW      1011       /* Acknowledge init new start */
#define EC_FLXSIM_ACK_RESUME   1012   /* Acknowledge init resumed start */
#define EC_FLXSIM_NACK         1019             /* Negative acknowledge */

#define EC_FLXSIM_SIGNDATA     1020        /* Autosignificantiegegevens */
#define EC_FLXSIM_SIGNREACHED  1021            /* Significantie bereikt */

#define EC_FLXSIM_QUIT_TRFFIL  1030    /* Quit traffic file (not valid) */


/* event codes flxrgl */
/* ------------------ */
#define EC_FLXRGL_NEW          2000                   /* Init new start */
#define EC_FLXRGL_RESUME       2001               /* Init resumed start */
#define EC_FLXRGL_START        2002                     /* Do start, go */
#define EC_FLXRGL_STOP         2003                             /* Stop */

#define EC_FLXRGL_ACK_NEW      2010       /* Acknowledge init new start */
#define EC_FLXRGL_ACK_RESUME   2011   /* Acknowledge init resumed start */
#define EC_FLXRGL_NACK         2019             /* Negative acknowledge */

#define EC_FLXRGL_TRACE        2020                            /* Trace */
#define EC_FLXRGL_ERROR        2021            /* Computer error FLXRGL */

#define EC_FLXRGL_SGTYPE       2030                /* Signaalgroep type */


/* signaalgroep status codes */
/* ------------------------- */
#define EXT_ROOD                  0
#define EXT_GROEN                 1
#define EXT_GEEL                  2
#define EXT_KNIPWIT               3
#define EXT_GEDOOFD               4
#define INT_BASE                256
#define INT_GL                  256
#define INT_VGL                 257
#define INT_VAGL                258
#define INT_WR                  266
#define INT_ROG                 268
#define INT_RVG                 270
#define INT_VG                  277
#define INT_VAG_1e              278
#define INT_WG                  279
#define INT_VAG_2e              280
#define INT_MVG                 281
#define MTM_BASE                512
#define MTM_GEDOOFD             512
#define MTM_KRUIS               513
#define MTM_PIJL                514
#define MTM_30                  515
#define MTM_40                  516
#define MTM_50                  517
#define MTM_60                  518
#define MTM_70                  519
#define MTM_80                  520
#define MTM_90                  521

/* event codes rwscreg */
/* ------------------- */
#define EC_SGSTAT             3000   /* Actuele status van signaalgroep */
#define EC_SET_DTSTAT         3010   /* Set actuele status van detector */
#define EC_DTSTAT             3011        /* Atuele status van detector */
#define EC_FASEBEWAKING       3020         /* Fasebewaking aangesproken */
#define EC_CREGITEM           3030              /* C-regelaar item data */
#define EC_SGFC_EXTERN        3040      /* Send only external SG status */
#define EC_SGFC_INTERN        3041      /* Send only internal SG status */

#define EC_CR_ENABLE          3090   /* Enable regelaar (levensteken=1) */
#define EC_CR_DISABLE         3091  /* Disable regelaar (levensteken=0) */

#define EC_KOPPEL_GETNOFREG   3100 /* Vraag aantal gekoppelde regelaars */
#define EC_KOPPEL_GETREGDATA  3101/* Vraag gekoppelde regelaar gegevens */

#define EC_KOPPEL_ENABLEREG   3110       /* Enable a coupled controller */
#define EC_KOPPEL_DISABLEREG  3111      /* Disable a coupled controller */

#define EC_KOPPEL_ENABLECOM   3120        /* Enable koppel communicatie */
#define EC_KOPPEL_DISABLECOM  3121       /* Disable koppel communicatie */
#define EC_KOPPEL_COMENABLED  3125        /* Koppel communicatie is aan */
#define EC_KOPPEL_COMDISABLED 3126        /* Koppel communicatie is uit */

#define EC_KOPPEL_SIGNAL      3130                   /* A koppel signal */

#define EC_KOPPEL_INIT        3140 /* Coupled controller is initialized */

#define EC_CR_DUMP            3150                /* Create a dump file */
#define EC_CR_DUMP_CREATED    3151                 /* Dump file created */

#define EC_CR_BLOCKINFO       3160         /* Active block info changed */

#define EC_CR_SET_STATUS      3200    /* Set the status of the regelaar */
#define EC_CR_STATUS          3201       /* The current regelaar status */
#define EC_CR_GET_STATUS      3202   /* Get the current regelaar status */

#define EC_CR_FIXEER          3210           /* (Re)set the fixeer flag */

#define EC_PRINTXY_UPDATE     3300             /* Printxy window update */

/* C-regelaar status */
/* ----------------- */
#define CR_STAT_ONGEDEF       -1            /* Ongedefinieerde toestand */
#define CR_STAT_INIT           0            /* Initialiseer de regelaar */
#define CR_STAT_REGEL          1                      /* Gewoon regelen */
#define CR_STAT_EXIT           5                  /* Beeindigen regelen */
#define CR_STAT_HALTEER        8                 /* Halteer de regelaar */
#define CR_STAT_HERINIT       10        /* Her-initializeer de regelaar */
#define CR_STAT_GEDOOFD       11         /* Alle signaalgroepen gedoofd */
#define CR_STAT_GEELKNIP      12      /* Alle signaalgr. geel knipperen */
#define CR_STAT_GEEL          13     /* Signaalgr. geel bij inschakelen */
#define CR_STAT_ROOD          14            /* Alle signaalgroepen rood */

/* event codes crdebug */
/* ------------------- */
#define EC_CRDEBUG_ITEM       4000                   /* Debug item info */
#define EC_CRDEBUG_UPDATE     4300               /* Update debug window */


/* -------------------------------------------------------------------- */
/* TYPE DEFINITIONS                                                     */
/* -------------------------------------------------------------------- */

typedef long mid_t;                            /* Module identification */

typedef long cod_t;                                /* Message code type */

typedef long st_t;                                    /* Simulated time */

typedef long siz_t;                                        /* Size type */

typedef unsigned long idx_t;                              /* Index type */

typedef struct tagrt_t                                     /* Real time */
{
   short year;                                             /* e.g. 1996 */
   short month;                               /* January=0, December=11 */
   short day;                                    /* Day of month (0-31) */
   short daytype;                               /* Sunday=0, Saturday=6 */
   short hour;                                          /* Hours (0-23) */
   short min;                                         /* Minutes (0-59) */
   short sec;                                         /* Seconds (0-59) */
   short msec;                                 /* Milli-seconds (0-999) */
} rt_t;
                                                                                                           /* Event handler */
typedef void (*hdl_t)(mid_t from, mid_t to, st_t time, cod_t code, siz_t size,
                      void* data);

typedef struct tagmsg_t                               /* Message object */
{
   mid_t from;
   mid_t to;
   st_t  time;
   cod_t code;
   siz_t size;
   void* data;
   idx_t idata;
   idx_t num;
} msg_t;

typedef void (*mon_t)(msg_t* msg);                      /* Monitor hook */


/* EVENT DATA */
/* ---------- */

/* flxsim */
/* ------ */
typedef struct tagflxsiminit_t                      /* Init FLXSIM data */
{
   char* flash_dir;                                  /* FLASH directory */
   char* work_dir;                   /* Working directory (with NETFIL) */
   char* simdata_name;            /* Simulation data (SIMDAT) file name */
} flxsiminit_t;

typedef struct tagflxsimackinit_t       /* Acknowledge FLXSIM init data */
{
   long resumeflag;                  /* Resume flag (0=new or 1=resume) */
   long resumetime;                                      /* Resume time */
} flxsimackinit_t;

typedef struct tagflxsimnew_t                        /* New FLXSIM data */
{
   char* netfil_name;                               /* NETFIL file name */
   char* confil_name[999];                         /* CONFIL file names */
   long  traceflag;                                       /* Trace flag */
} flxsimnew_t;

typedef struct tagflxsimresume_t                  /* Resume FLXSIM data */
{
   long traceflag;                                        /* Trace flag */
} flxsimresume_t;

typedef struct tagflxsimsubrun_t                    /* Subrun completed */
{
   long numofruns;                                       /* aantal runs */
} flxsimsubrun_t;

typedef struct tagsigndata_t            /* Simulation significancy data */
{
   float sigval[8];
   long  numofruns[8];
   float standdev[8];
   long  numofcars[8];
} flxsimsigndata_t;


/* flxrgl */
/* ------ */
typedef struct tagflxrglsgtype_t            /* Signaalgroep status data */
{
   short kpr_nr;                           /* Kruispuntsregeling nummer */
   short sg_nr;                                  /* Signaalgroep nummer */
   short type;                                     /* Signaalgroep type */
} flxrglsgtype_t;

typedef struct tagflxrglnew_t                         /* New simulation */
{
   char* flash_dir;                                  /* FLASH directory */
   char* work_dir;                   /* Working directory (with CONFIL) */
   char* confil_name;                                /* Confil filename */
} flxrglnew_t;

typedef struct tagflxrglresume_t                   /* Resume simulation */
{
   char* flash_dir;                                  /* FLASH directory */
   char* work_dir;                   /* Working directory (with CONFIL) */
} flxrglresume_t;

typedef struct tagflxrgltrace_t                          /* start trace */
{
   long trace;                                         /* trace options */
} flxrgltrace_t;

typedef struct tagflxrglsubrun_t                    /* Subrun completed */
{
   long numofruns;                                       /* aantal runs */
} flxrglsubrun_t;


/* rwscreg */
/* ------- */
typedef struct tagsgstat_t                  /* Signaalgroep status data */
{
   short kpr_nr;                           /* Kruispuntsregeling nummer */
   short sg_nr;                                  /* Signaalgroep nummer */
   short status;                                 /* Signaalgroep status */
} sgstat_t;

typedef struct tagdtstat_t                      /* Detector status data */
{
   short kpr_nr;                           /* Kruispuntsregeling nummer */
   short sg_nr;                                  /* Signaalgroep nummer */
   short dt_nr;                                  /* Detector volgnummer */
   short dt_type;                                      /* Detector type */
   short status;                                     /* Detector status */
} dtstat_t;

typedef struct tagkoppelreg_t               /* Gekoppelde regelaar data */
{
   short index;                 /* Index in gekoppelde regelingen array */
   short kpr_nr;              /* Kruispuntsregeling nummer van regelaar */
   short comflag;   /* Koppelcommunicatie met regeling is aanwezig vlag */
} koppelreg_t;

typedef struct tagkoppelsignal_t                  /* Koppelsignaal data */
{
   short kpr_nr;              /* Kruispuntsregeling nummer van afzender */
   short index;                                 /* Koppelsignaal nummer */
   short status;                                /* Koppelsignaal status */
} koppelsignal_t;

typedef struct tagblockinfo_t                        /* Block info data */
{
   short kpr_nr;              /* Kruispuntsregeling nummer van afzender */
   short block_nr;                               /* Active block number */
} blockinfo_t;


/*----------------------------------------------------------------------*/
/* FUNCTION PROTOTYPES                                                  */
/*----------------------------------------------------------------------*/

/* Event dispatcher */
/* ---------------- */
int   WINAPI msg_hdl_set(hdl_t hdl, mid_t id); /* Install mssge handler */
int   WINAPI msg_hdl_rls(mid_t id);        /* Release a message handler */
int   WINAPI msg_put(mid_t from, mid_t to, st_t time, cod_t code, siz_t size,
                     void* data);
int   WINAPI msg_mode(int mode);         /* Set the event dispatch mode */
mon_t WINAPI mon_set(mon_t mon);    /* Install/release monitor function */


/* Simulated time clock */
/* -------------------- */
int  WINAPI st_init(st_t now, st_t gran, st_t mul); /* Init. clock mod. */
int  WINAPI st_exit(void);                     /* Exit the clock module */
int  WINAPI st_mode(int mode); /* Change operation mode of clock module */
int  WINAPI st_mul(st_t mul);   /* Change time-step multiply factor (%) */
st_t WINAPI st_set(st_t now);               /* Set a new simulated time */
st_t WINAPI st_get(st_t offset);      /* Get the current simulated time */

int  WINAPI rt_set(rt_t* rt);              /* Set the current real time */
int  WINAPI rt_get(rt_t* rt);              /* Get the current real time */


#ifdef __cplusplus
}
#endif


