#ifndef CRDTO_D
#define CRDTO_D

/*
 * Copyright(c) 1992 RWS Dienst Verkeerkunde
 *
 * File         : crdto.d
 *
 * Description  : datastructures in het detector object
 *
 * History      : 05-02-90 Creation
 *                6-7-92,  aanpassing aan nieuwe interface definitie
 *                         van de commissie C van de CVN
 *
 */

/*
 * het maximum aantal detectoren bedraagt MAX_DET (=CIF_AANT_IS_D).
 * Verdeeld in groepen van 16.
 * het aantal groepen is bepaald in MAX_DTGR
 *
 */

#define         MAX_DTGR        ((MAX_DET+15)/16)

/*
 * Deze eerste structure bevat elementen voor de tijden.
 *
 */

struct DT_BASE
{
  Bool16  d_td [ MAX_DET ];           /* tijden */
};


/*
 * deze structure bevat applicatie informatie die per DT is uitgegeven.
 *
 */
struct DT_PARM
{
  Bool16  keuze_eh;              /* extra hiaattijd gekozen */
};


struct DT_PTRS {
        Int16   sg_num;          /* bijbehorend SG nummer */
        Mask16  type;            /* BF, DK, DR, DS ) */
        Mask16  m_functie;       /* bitmasker met gewenste functies */
        Mask16  functie;         /* bitmasker met de volgende info:
                                         - aanvraag functie
                                         - benvloedt 1e hiaattijd
                                         - benvloedt 2e hiaattijd
                                         - 1e in richtinggevoelige aanvraag
                                         - 2e in richtinggevoelige aanvraag 
                                         - benvloedt Basisspec Hoofdst. 5 formule p. */
        Bool16  m;               /* detector ID */
        Bool16  *s;              /* pointer naar detectorstanden */
        Bool16  *vs;             /* pointer naar de vorige detectorstanden */
        Bool16  *s_s;            /* pointer naar detectorstanden */
        Bool16  *vs_s;           /* pointer naar de vorige detectorstanden */
        Bool16  *s_og;           /* pointer naar detectorstanden */
        Bool16  *vs_og;          /* pointer naar de vorige detectorstanden */
        Bool16  *s_bg;           /* pointer naar detectorstanden */
        Bool16  *vs_bg;          /* pointer naar de vorige detectorstanden */
        Bool16  *s_fg;           /* pointer naar detectorstanden */
        Bool16  *vs_fg;          /* pointer naar de vorige detectorstanden */
        Mask16  tr;              /* bitmasker met de volgende info:
                                    -  1 = tr info d
                                    -  2 = tr info d_s
                                    -  4 = tr info d_og
                                    -  8 = tr info d_bg
                                    - 16 = tr info d_fg   */

        Bool16* b_td;            /* pointers naar waarden uit deze ronde */
        Bool16* bg_td;
        Bool16* en_td;
        Bool16* tr_td;
        Bool16* tr_wtd;
        Bool16* hl_td;

        /* pointers naar booleans uit de configuratietabel     */
        Bool16* p_eh;

        /* pointers naar tijdselementen uit de timer structure */
        Tijdelement *p_dt [ MAX_D_TD ];
        };

struct DT_GROEP {
        Bool    bs_h5p;         /* BS hoofdstuk 5 formule p */
        Int16   h1;             /* aantal detectoren met H1e functie die op zijn */
        Int16   h2;             /* aantal detectoren met H2e functie die op zijn */
        Int16   h1def;          /* aantal detectoren met H1e functie en gedefinieerde hiaattijd */
        Int16   h2def;          /* aantal detectoren met H2e functie en gedefinieerde hiaattijd */
        Int16   d;              /* aantal detectoraanvragen */
        Int16   df;             /* aantal detectoraanvragen */
        Int16   dk;             /* aantal detectoraanvragen */
        Int16   ds;             /* aantal detectoraanvragen */
        };

#endif  /* CRDTO_D */
