#ifndef CRSGO_D
#define CRSGO_D

/*
 * Copyright(c) 1992 RWS Dienst Verkeerkunde
 *
 * File         : crsgo.d
 *
 * Description  : datastructures in het signaalgroep object
 *
 * History      : 18-01-90 Creation
 *                6-7-92,  aanpassing aan nieuwe interface definitie
 *                         van de commissie C van de CVN
 *
 */

/*
 * Deze eerste structure bevat elementen voor de toestanden, de
 * kleuren, de tijden ( muv de ontruimingstijden ), blok-
 * onafhankelijke hulpfuncties en blok-onafhankelijke opties.
 *
 */

struct SG_BASE
{
  Bool16  brt;                     /* beurt voor de sg */
  Bool16  hf [ MAX_HFS ];          /* blok-onafhankelijke hulpfuncties */
                                   /* inclusief de zeven HFMAn hulpfuncties  */
                                   /* en de HFRGAn hulpfunctie uit hoofdstuk 7 */
  Bool16  op [ MAX_OPS ];          /* blok-onafhankelijke opties */
  Bool16  ts [ MAX_TS ];           /* signaalgroep toestanden */
                                   /* inclusief fictieve SG toestand voor gekoppelde SG's */
  Bool16  td [ MAX_TD ];           /* tijden , inclusief TIJDn,1 t/m TIJDn,5 */
                                   /* uit hoofdstuk */
  Bool16  ot [ MAX_SG ];           /* ontruimingstijden van andere SG's naar deze SG */
  Bool16  got [ MAX_SG ];          /* garantie-ontruimingstijden */
};


/*
 * Deze tweede structure bevat user defined hulpfuncties, niet uitgegeven per 
 * SG of per blok (inclusief HFWSR uit hoofdstuk 7).
 *
 */
struct SG_UHF
{
  Bool    hf[ MAX_HF ];
};

/*
 * deze structure bevat applicatie informatie die per SG is uitgegeven.
 *
 */
struct SG_PARM
{
  Bool16  voet;                    /* wel / geen voetganger/fietsrichting */
  Bool16  conf [ MAX_SG ];         /* conflict matrix          */
  Bool16  fict [ MAX_SG ];         /* fictieve conflict matrix */
  Bool16  h5_t [ MAX_SG ];         /* matrix volgens BS_H5_t   */
  Bool16  h5_ac[ MAX_SG ];         /* matrix volgens BS_H5_ac  */
  Bool16  h66  [ MAX_SG ];         /* matrix gekoppelde SG's   */
  Bool16  wsr;                     /* wachtstandrichting       */
};

/*
 * Deze structure wordt per SG uitgegeven en bevat alleen maar pointers
 *
 */
struct SG_PTRS
{
  /* bitmasker voor selectie 1 uit 16 en       */
  /* bitmasker voor de toestand                */
  Mask16  m;
  Mask16  toestand;

  /* pointers naar waarden uit deze ronde      */
  Bool16* b_brt;
  Bool16* b_hf;
  Bool16* b_op;
  Bool16* b_ts;
  Bool16* b_td;
  Bool16* b_ot;
  Bool16* b_got;

  /* pointers naar waarden uit de vorige ronde */
  Bool16* bg_brt;
  Bool16* bg_hf;
  Bool16* bg_op;
  Bool16* bg_ts;
  Bool16* bg_td;
  Bool16* bg_ot;
  Bool16* bg_got;

  /* pointers naar waarden uit de vorige ronde */
  Bool16* en_brt;
  Bool16* en_hf;
  Bool16* en_op;
  Bool16* en_ts;
  Bool16* en_td;
  Bool16* en_ot;
  Bool16* en_got;

  /* pointers naar waarden uit de trace tabel  */
  Bool16* tr_brt;
  Bool16* tr_hf;
  Bool16* tr_op;
  Bool16* tr_ts;
  Bool16* tr_td;
  Bool16* tr_ot;
  Bool16* tr_got;
  Bool16* tr_wtd;

  /* pointers naar waarden uit de trace tabel  */
  Bool16* hl_brt;
  Bool16* hl_hf;
  Bool16* hl_op;
  Bool16* hl_ts;
  Bool16* hl_td;
  Bool16* hl_ot;
  Bool16* hl_got;

  /* pointers naar booleans uit de configuratietabel     */
  Bool16* p_v;
  Bool16* p_c;
  Bool16* p_f;
  Bool16* p_wsr;

  /* pointers naar tijdselementen uit de timer structure */
  Tijdelement *p_t [ MAX_TD ];
  Tijdelement *p_ot [ MAX_SG ];
  Tijdelement *p_got [ MAX_SG ];
};

#endif  /* CRSGO_D */
