#ifndef RWS_DIM_H
#define RWS_DIM_H

/*
 * Copyright(c) 1992 RWS Dienst Verkeerkunde
 *
 * File         : rws_dim.h
 *
 * Description  : systeemgrenzen van de RWS C-regelaar
 *
 * History      : Creation.
 *                6-7-92,  aanpassing aan nieuwe interface definitie
 *                         van de commissie C van de CVN
 *                11-12-95, aanpassing t.b.v. uitbreiding "overige" deel
 *                          van de buffers CIF_GUS, CIF_WUS en CIF_IS
 *
 */

/***************************************************************************/
/*  B R E N G    H I E R    V E R A N D E R I N G E N  A A N   O M   D I T */
/*  P A K K E T   A A N  U W   W E N S E N   A A N  T E   P A S S E N ! !  */
/*  CIF_AANT_US_FC  mag slechts de waarden: 16, 32, 48 of 64 hebben        */
/***************************************************************************/

#define   CIF_AANT_US_FC    48      /* signaalgroepen                  */
#define   RWS_AANT_BLOKKEN  10      /* blokken                         */
#define   RWS_AANT_KL       20      /* klokken                         */

/***************************************************************************/
/*  B R E N G    H I E R O N D E R   G E E N   V E R A N D E R I N G E N   */
/*                             A A N !                                     */
/***************************************************************************/

#define   OVERIGE_SIGNALEN 128    /* overige in-/uit-gangssignalen   */
#define   MAX_KOPPELSGN    128    /* aantal koppelsignalen           */
#define   MAX_REGELINGEN   10     /* maximum aantal gekoppelde regelingen */
#define   CIF_AANT_US_OV  (OVERIGE_SIGNALEN + MAX_REGELINGEN + 2*MAX_KOPPELSGN)
                                  /* overige uitgangen               */

#define   CIF_AANT_IS_D   256     /* detectoren                      */
#define   CIF_AANT_IS_OV  (OVERIGE_SIGNALEN + MAX_KOPPELSGN + CIF_AANT_IS_D + 2*MAX_REGELINGEN + CIF_AANT_IS_D)
                                  /* overige ingangen                */


#define   CIF_AANT_GPS    5       /* gewenste programmastatus        */
#define   CIF_AANT_WPS    5       /* werkelijke programmastatus      */
#define   CIF_MAX_UBER    1024    /* uitgaande berichten             */
#define   CIF_MAX_IBER    1024    /* ingaande berichten              */
#define   CIF_AANT_KLOK   10      /* klok                            */
#define   CIF_WEL_OT              /* ontruimingstijden in interface  */
                                  /* maatgevend voor procesbesturing */

#define   RWS_AANT_REG    1       /* regelaars (waarde voor flexsyt) */
#define   RWS_AANT_RGA    4       /* richting gevoelige aanvraag     */

#if (RWS_AANT_RGA > RWS_AANT_BLOKKEN)
#define   RWS_AANT_BK     RWS_AANT_RGA
#else
#define   RWS_AANT_BK     RWS_AANT_BLOKKEN
#endif

#define   CIF_AANT_DSI    36      /* selectieve detectie             */
#define   CIF_MAX_MON_IBER 1024   /* ingaande berichten monitoring   */
#define   CIF_MAX_MON_UBER 1024   /* uitgaande berichten monitoring  */

#define   CIF_MAX_COM_UBER  2048  /* uitgaande berichten COM poort   */
#define   CIF_MAX_COM_IBER  2048  /* ingaande berichten COM poort    */

#define   CIF_MAX_FILE_UBER 16384 /* uitgaande berichten fileopslag  */
#define   CIF_MAX_FILEEXT    4    /* file-extensie fileopslag        */

#endif  /* RWS_DIM_H */
