/*************************************************************************
*  Program Name  : FLASH (FLexit Application SHell)                      *
*========================================================================*
*                                                                        *
*  Filename      : rwscreg.c                                             *
*                                                                        *
*  Function      : C-Regelaar functions                                  *
*                                                                        *
*  Date Modified : March 2007                                            *
*                                                                        *
*  Modifications : Licentienummer en bedrijfsnaam weergeven in workbench *
*************************************************************************/

/* -------------------------------------------------------------------- */
/*  INCLUDED FILES                                                      */
/* -------------------------------------------------------------------- */

#include <windows.h>
#include <stdio.h>
#include <assert.h>
#include "rws_dim.h"
#include "crtypes.h"
#include "cif.inc"
#include "evtdisp.h"
#include "evtdintf.h"
#include "crfpbrt.h"
#include "rwscreg.h"

#ifdef LOG
FILE* logfile = NULL;
#endif

extern s_int16 set_lic_nr(void);
extern char* get_bedrijfsnaam(void);
extern Int16 versienummer_cr;

/* -------------------------------------------------------------------- */
/*  IMPLEMENTATION                                                      */
/* -------------------------------------------------------------------- */
extern char versienummer_pb[10];

/*----------------------------------------------------------------------*/
/* name     : rwscregInit                                               */
/* function : Initialize the RWS-C regelaar.                            */
/*                                                                      */
/* input    :                                                           */
/* output   :                                                           */
/* return   :                                                           */
/* remarks  :                                                           */
/*                                                                      */
/*----------------------------------------------------------------------*/

static int rwscregInit(HINSTANCE hInstance)
{
   assert (hInstance);

   if (evtdisp_lib_init() == 0)
   {
      cregInit(hInstance);
      if (msg_hdl_set((hdl_t)cregEventHandler, MID_CREG(KPRNR)) >= 0)
      {
#ifdef LOG
         assert (logfile == NULL);
         logfile = fopen("RWSCREG.LOG", "wt");
#endif
         /* stuur licentienummer en bedrijfsnaam van de gebruiker naar de workbench */
         char text[128];
         sprintf(text, "Kruispuntnummer %d : Procesbesturing.Basisstructuur versie %s.%d",
                       KPRNR, versienummer_pb, versienummer_cr );
         msgDisplay( EC_MSG_TEXT, text );
         sprintf(text, "Licentienummer %d : %s", set_lic_nr(), get_bedrijfsnaam() );
         msgDisplay( EC_MSG_TEXT, text );

         return (TRUE);
      }
      msgDisplay(EC_MSG_ERROR, "De C-regelaar is reeds opgestart.");
      evtdisp_lib_exit();
   }
   MessageBox (NULL, "ERROR : De C-regelaar kon niet worden opgestart.",
               "RWSCREG.DLL", MB_OK);
   return (FALSE);
}                                                        /* rwscregInit */


/*----------------------------------------------------------------------*/
/* name     : rwscregExit                                               */
/* function : Exit the RWS-C regelaar.                                  */
/*                                                                      */
/* input    :                                                           */
/* output   :                                                           */
/* return   :                                                           */
/* remarks  :                                                           */
/*                                                                      */
/*----------------------------------------------------------------------*/

static void rwscregExit(void)
{
   cregExit();
   msg_hdl_rls(KPRNR);
   evtdisp_lib_exit();
#ifdef LOG
   if (logfile)
   {
      fclose(logfile);
      logfile = NULL;
   }
#endif
}                                                        /* rwscregExit */


#ifdef __WCC__                                       /* Watcom compiler */

/*----------------------------------------------------------------------*/
/* name     : LibMain                                                   */
/* function : Main function of DLL.                                     */
/*                                                                      */
/* input    : hInstance   = Current instance handle                     */
/*            wDataSeg    = Data segment                                */
/*            cbHeapSize  = Heap size                                   */
/*            lpszCmdLine = Command line                                */
/* output   :                                                           */
/* return   : 1 (successful)                                            */
/* remarks  :                                                           */
/*                                                                      */
/*----------------------------------------------------------------------*/

int __export CALLBACK LibMain(HINSTANCE hInstance, WORD wDataSeg,
                              WORD cbHeapSize, LPSTR lpszCmdLine)
{
   assert (hInstance == hInstance);
   assert (wDataSeg == wDataSeg);
   assert (cbHeapSize == cbHeapSize);
   assert (lpszCmdLine == lpszCmdLine);

   return (rwscregInit(hInstance));
}                                                            /* LibMain */


/*----------------------------------------------------------------------*/
/* name     : WEP                                                       */
/* function : Windows Exit Procedure                                    */
/*                                                                      */
/* input    : nExitType = Exit type                                     */
/* output   :                                                           */
/* return   : 1 (successful)                                            */
/* remarks  :                                                           */
/*                                                                      */
/*----------------------------------------------------------------------*/

int __export CALLBACK WEP(int nExitType)
{
   assert (nExitType == nExitType);

   rwscregExit();
   return (TRUE);
}                                                                /* WEP */


#else                            /* Presume Microsoft Visual C compiler */


/*----------------------------------------------------------------------*/
/* name     : DllMain                                                   */
/* function : Main function of DLL.                                     */
/*                                                                      */
/* input    : hInstance   = Current instance handle                     */
/*            dwReason    = Reason for calling this function            */
/*            lpReserved  = Reserved pointer for data                   */
/* output   :                                                           */
/* return   : TRUE (successful)                                         */
/* remarks  :                                                           */
/*                                                                      */
/*----------------------------------------------------------------------*/

BOOL WINAPI DllMain(HINSTANCE hInstance, DWORD dwReason, LPVOID lpReserved)
{
   assert (hInstance == hInstance);
   assert (dwReason == dwReason);
   assert (lpReserved == lpReserved);

   switch (dwReason)
   {
       case DLL_PROCESS_ATTACH:
          return (rwscregInit(hInstance));

       case DLL_PROCESS_DETACH:
          rwscregExit();
          break;

       default:
          break;
   }
   return (TRUE);
}                                                            /* DllMain */


#endif
